#!/bin/sh

BUILD_STARTTIME=`date "+%Y-%m-%d %H:%M:%S"`
echo "*******************编译开始时间: ${BUILD_STARTTIME}"
BUILD_STARTTIME=`date +%s`


#打印执行的命令，只支持单参 
function echoCommand()
{
    echo "[******************* Start excuting command:$1]"
    $1
    echo "[******************* End excuting command...] \r\n"
}

#打印xcode、编译环境信息
function printXcodeInfo()
{
    xcode-select --version
    xcode-select --print-path
    security find-identity -v -p codesigning

    echo "Python version:"
    python --version

    # echo "Pod version:"
    # pod --version

    # echo "Remove Cache"
    # rm -rf ~/Library/Developer/Xcode/DerivedData/

	# echo "Set PBXNumberOfParallelBuildSubtasks 8"
	# defaults write com.apple.Xcode PBXNumberOfParallelBuildSubtasks 8
	# defaults write com.apple.dt.Xcode ShowBuildOperationDuration YES
}

#打印授权文件hash及名称
function printProvisionFiles()
{
    for file in `ls ~/Library/MobileDevice/Provisioning\ Profiles/`; do
    	echo "$(/usr/bin/security cms -D -i ~/Library/MobileDevice/Provisioning\ Profiles/$file)" > "temp_plist.plist"
        echo "$file, ExpirationDate: `/usr/libexec/PlistBuddy -c "Print ExpirationDate" temp_plist.plist`, Name: `/usr/libexec/PlistBuddy -c "Print Name" temp_plist.plist`"
        #datetime=`/usr/libexec/PlistBuddy -c "Print ExpirationDate" temp_plist.plist`
        rm -rf temp_plist.plist
    done
}

# Generate entitlements
# 通过Profile文件生成签名用的entitlements.plist文件
#参数1：Profile文件，保存至ENTITLEMENTS_PLIST中
#返回值：plist文件路径
function generateEntitlementPlistFile()
{
	if [[ -z $1 ]]; then
		echo "Error: No profiles input..."
	fi

	provisionvalue=`cat "${1}"`
	parseEntitlement=${provisionvalue#*<key>Entitlements</key>}
	entitlementFromMPP=${parseEntitlement%%</dict>*}
	entitlementHeader1='<?xml version="1.0" encoding="UTF-8"?>'
	entitlementHeader2='<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">'
	entitlementHeader3='<plist version="1.0">'
	fullEntitlement=$entitlementHeader1$entitlementHeader2$entitlementHeader3"${entitlementFromMPP}</dict></plist>"
	echo "${fullEntitlement}" > "$(pwd)/entitlements.plist"

	#echo "------------ Entitlements file used --------------"
	#echo "${fullEntitlement}"
	#echo "--------------------------------------------------"

	echo "$(pwd)/entitlements.plist"
}

#对可执行文件进行签名
#参数1：授权文件路径
#参数2：证书KeychainId
#参数3：可执行文件路径
function resignFile()
{
	echo "Resign File: $1, $2, $3"
	entitlementsPlist=`generateEntitlementPlistFile $1`

	#去除旧的签名
	echo "Remove _CodeSignature..."
	rm -rf "$3/_CodeSignature"

	#拷贝描述文件
	echo "Copy provisioning file to ... $3/embedded.mobileprovision"
	cp -rf "$1" "$3/embedded.mobileprovision"

	#目录下有Frameworks文件夹，则需要对所有动态库进行重签名
	if [ -d "$3/Frameworks" ];then
		`codesign -v -f -s $2 $3/Frameworks/*`
	fi

	#对可执行文件进行签名
	`codesign -v -f -s $2 --entitlements ${entitlementsPlist} $3`
}

#检查文件路径是否存在，如果不存在，直接结束
#参数1：文件路径
function checkFile() {
	if [[ -z $1 ]]; then
		echo "Check file with nothing..."
	fi

	if [[ ! -f $1 ]]; then
		echo "[******************** Checking file failed: $1, now ending... ********************] \r\n \r\n"
		exit
	fi
}

#从目录中查找ipa文件
#参数1：文件路径
function findIPAFileInFolder() {
	for file in `find $1 name "*.ipa"`
	do
    	echo "Find file: ${file}"
	done
}

# 打印Xcode、证书、授权文件相关信息
echoCommand "printXcodeInfo"
echoCommand "printProvisionFiles"

# 工程路径、打包文件名相关的配置
echo "[******************** Set Project Parameters ********************]"
#打包脚本所在路径
BUILD_SCRIPT_PATH=`pwd`
echo "BUILD_SCRIPT_PATH: $BUILD_SCRIPT_PATH"

#打包的workspace文件的路径
WORKSPACE_FILE_PATH=../LeChangeDemo.xcworkspace
echo "WORKSPACE_FILE_PATH: ${WORKSPACE_FILE_PATH}"

#target名称
SCHEME_NAME=LeChangeDemo
echo "SCHEME_NAME: ${SCHEME_NAME}"

#项目plist文件
PLIST_FILE_PATH=../LeChangeDemo/Info.plist
echo "PLIST_FILE_PATH: ${PLIST_FILE_PATH}"

#SVN路径对应的文件夹名称，Jenkins打PROJECT_NAME包时需要对应，如iEasy4ip、iLeChange
PROJECT_NAME=iLeChange
echo "PROJECT_NAME: ${PROJECT_NAME}"

#IPA对应的程序名称
IPA_APP_NAME=LeChangeDemo
echo "IPA_APP_NAME: ${IPA_APP_NAME}"

echo "[******************** Set Project Parameters End ********************] \r\n \r\n"


# 证书、BundleId、授权文件相关配置
echo "[******************** Set Certificate and Provision Parameters ********************]"
#AppStore证书名称，在导出AppStore版本需要
PRODUCT_BUNDLE_IDENTIFIER='com.dahuatech.leopen'

AD_HOC_PROFILE_NAME=lechange_opensdkdemo_adhoc
AD_HOC_PROFILE_FILE="`pwd`/provisionfile/adhoc/lechange_opensdkdemo_adhoc.mobileprovision"

APP_STORE_PROFILE_NAME=lechange_opensdkdemo_appstore
APP_STORE_PROFILE_FILE="`pwd`/provisionfile/appstore/lechange_opensdkdemo_appstore.mobileprovision"

echoCommand "checkFile ${APP_STORE_PROFILE_FILE}"
echoCommand "checkFile ${AD_HOC_PROFILE_FILE}"

#发布版证书签名及证书路径，导出AdHoc版本需要
DISTRIBUTION_KEYCHAI_ID="AB41ADC686B7D60946D06DD7B21246C85821579A"


#导出IPA选用的方法: 'app-store'、'enterprise'、'ad-hoc'
# EXPORT_METHOD='enterprise'
EXPORT_METHOD='app-store'
echo "EXPORT_METHOD: ${EXPORT_METHOD}"



echo "[******************** Set Certificate and Provision Parameters End ********************] \r\n \r\n"


echo "[******************** Auto Parameters ********************]"

#获取BundleVersion: 结项时，版本号需要固定
BUNDLE_VERSION=$(python `pwd`/getVersion.py ${PLIST_FILE_PATH})
IPA_BUNDLE_VERSION=${BUNDLE_VERSION}
IPA_BUNDLE_VERSION='1.000.0000000.0' #$BUNDLE_VERSION
echo "BUNDLE_VERSION: $BUNDLE_VERSION"

#编译日期
BUILD_DATE=`date "+%Y%m%d"`
echo "BUILD_DATE: $BUILD_DATE"

#计算带日期的版本号
BUNDLE_VERSION_WITH_DATE=${BUNDLE_VERSION}.$(date +%m%d)
echo "BUNDLE_VERSION_WITH_DATE:"${BUNDLE_VERSION_WITH_DATE}

#获取工程所在路径
SRCROOT=${WORKSPACE_FILE_PATH%/*}
echo "SRCROOT:"${SRCROOT}

WORKSPACE_FILE_NANE=${WORKSPACE_FILE_PATH##*/}
echo "WORKSPACE_FILE_NANE:"${WORKSPACE_FILE_NANE}

WORKSPACE_NAME=${WORKSPACE_FILE_NANE%.*}
echo "WORKSPACE_NAME:"${WORKSPACE_NAME}

echo "Enter Project Dir"
cd ${SRCROOT}

#记录绝对路径：工程路径
PROJECT_DIR=$(pwd)
echo "PROJECT_DIR:"${PROJECT_DIR}

echo "[********************  Auto Parameters End ********************] \r\n \r\n"


echo "[******************** Step1. Start Packaging $(date +%Y-%m-%d\ %H:%M:%S) ********************]"
ARCHIVE_NAME=archivePath
echo "ARCHIVE_NAME:"${ARCHIVE_NAME}
ARCHIVE_PATH=${PROJECT_DIR}/${ARCHIVE_NAME}.xcarchive
echo "ARCHIVE_PATH:"${ARCHIVE_PATH}
xcodebuild archive -workspace ${WORKSPACE_FILE_NANE} -configuration clean -scheme ${SCHEME_NAME} -archivePath ${ARCHIVE_PATH}
echo "[******************** End Packaging $(date +%Y-%m-%d\ %H:%M:%S) ********************] \r\n \r\n"


echo "[******************** Start Checking Archive ********************]"
echo "Checking archive at path:${ARCHIVE_PATH}"
#判断导出的archive是否存在，不存在直接结束
if [ ! -d "${ARCHIVE_PATH}" ];then
    echo "[******************** Building failed, now ending... ********************] \r\n \r\n"
    exit
fi
echo "[******************** End Checking Archive ********************] \r\n \r\n"


#=============使用AppStore的授权文件导出IPA包=================
echo "[******************** Step2. Export ipa ... ********************]"
ENTITLEMENTS_PLIST=`generateEntitlementPlistFile "${APP_STORE_PROFILE_FILE}"`
DEVELOPMENT_TEAM=$(/usr/libexec/PlistBuddy -c "print :com.apple.developer.team-identifier" "$ENTITLEMENTS_PLIST")
echo "DEVELOPMENT_TEAM: $DEVELOPMENT_TEAM"

applestore_plist_store=$'<?xml version="1.0" encoding="UTF-8"?>\n'
applestore_plist_store=${applestore_plist_store}$'<!DOCTYPE plist PUBLIC "-//Apple//DTD PLIST 1.0//EN" "http://www.apple.com/DTDs/PropertyList-1.0.dtd">\n'
applestore_plist_store=${applestore_plist_store}$'<plist version="1.0">\n'
applestore_plist_store=${applestore_plist_store}$'<dict>\n'
applestore_plist_store=${applestore_plist_store}$'	<key>uploadSymbols</key>\n'
applestore_plist_store=${applestore_plist_store}$'	<true/>\n'
applestore_plist_store=${applestore_plist_store}$'	<key>method</key>\n'
applestore_plist_store=${applestore_plist_store}$'	<string>'${EXPORT_METHOD}'</string>\n'
applestore_plist_store=${applestore_plist_store}$'	<key>teamID</key>\n'
applestore_plist_store=${applestore_plist_store}$'	<string>'${DEVELOPMENT_TEAM}$'</string>\n'
applestore_plist_store=${applestore_plist_store}$'	<key>provisioningProfiles</key>\n'
applestore_plist_store=${applestore_plist_store}$'	<dict>\n'
applestore_plist_store=${applestore_plist_store}$'		<key>'${PRODUCT_BUNDLE_IDENTIFIER}$'</key>\n'
applestore_plist_store=${applestore_plist_store}$'		<string>'${APP_STORE_PROFILE_NAME}$'</string>\n'

applestore_plist_store=${applestore_plist_store}$'	</dict>\n'
applestore_plist_store=${applestore_plist_store}$'</dict>\n'
applestore_plist_store=${applestore_plist_store}$'</plist>'

APP_STORE_PLIST_FILE=applestore_temp.plist
echo "${applestore_plist_store}" > "${APP_STORE_PLIST_FILE}"
echo "Export ipa with entitlements: ${APP_STORE_PLIST_FILE}"

xcodebuild -exportArchive -archivePath ${ARCHIVE_PATH} -exportOptionsPlist ${APP_STORE_PLIST_FILE} -exportPath ipa-build

EXPORT_DIR=${PROJECT_DIR}/ipa-build
echo "EXPORT_DIR:"${EXPORT_DIR}
mkdir ${EXPORT_DIR}

#找到导出后的ipa文件路径，有些设置不正确时，导致的IPA并不是SCHEME_NAME.ipa，直接从archivePath.xcarchive读取ipa名称
IPANAME=$(/usr/libexec/PlistBuddy -c "print :Name" "${ARCHIVE_PATH}/Info.plist")
echo "Find ipa name: $IPANAME"
EXPORT_IPA_FILE_PATH=${EXPORT_DIR}/${IPANAME}.ipa

echo "EXPORT_IPA_FILE_PATH:"${EXPORT_IPA_FILE_PATH}


#检验IPA是否导出成功
checkFile ${EXPORT_IPA_FILE_PATH}

echo "[******************** End Export ipa... ********************] \r\n \r\n"


 #=============IPA包解压缩到tmp文件夹中=================
 echo "[******************** Step3. Unzip ipa ... ********************]"

 TEMP="${PROJECT_DIR}/temp"

 if [ ! -d $TEMP ]
 then
    mkdir -p $TEMP
 fi
 echo "TEMP:"$TEMP

 echo "start unzip ipa file, path: $EXPORT_IPA_FILE_PATH"
 unzip "$EXPORT_IPA_FILE_PATH" -d $TEMP

 #导出后  删除原来的ipa
 rm -rf $EXPORT_IPA_FILE_PATH

 APP_NAME=$(ls $TEMP/Payload)
 echo "AppName: $APP_NAME"

 TEMP_UNZIP_INFOPLIST_FILE_PATH="${TEMP}/Payload/${APP_NAME}/Info.plist"
 echo "TEMP_UNZIP_INFOPLIST_FILE_PATH: ${TEMP_UNZIP_INFOPLIST_FILE_PATH}"
 echo "[******************** End Unzip ipa... ********************] \r\n \r\n"


# # build version 设置成时间格式 不用每次修改了
# BUILD_VERSION=$(date +%Y.%m%d.%H%M)

# `/usr/libexec/PlistBuddy -c "Set :CFBundleVersion $BUILD_VERSION" "$TEMP_UNZIP_INFOPLIST_FILE_PATH"`

 echo "[******************** Step4-1. Resign ipa for app-store... ********************]"

 #对App进行签名设置
 REGISN_PROFILE_FILE=$APP_STORE_PROFILE_FILE
 REGISN_KEYCHAI_ID=$DISTRIBUTION_KEYCHAI_ID
 REGISN_OUT_IPA=$EXPORT_DIR/General_${IPA_APP_NAME}App_Chn_AppStore_IS_V${IPA_BUNDLE_VERSION}.R.${BUILD_DATE}.ipa

 resignFile "$REGISN_PROFILE_FILE" "$REGISN_KEYCHAI_ID" "${TEMP}/Payload/${APP_NAME}"

 #生成ipa文件
 echo "zip file generate new ipa file"
 cd $TEMP

 #上传appstore的文件需要对应的SwiftSupport 和 Symbols
 echo `zip -qr resign.ipa Payload SwiftSupport Symbols`
 mv resign.ipa ${REGISN_OUT_IPA}

 echo "[******************** End Resign ipa for app-store... ********************] \r\n \r\n"


 echo "[******************** Step4-2. Resign ipa for ad-hoc... ********************]"
 
 #对App进行签名设置
 REGISN_PROFILE_FILE=$AD_HOC_PROFILE_FILE
 REGISN_KEYCHAI_ID=$DISTRIBUTION_KEYCHAI_ID
 REGISN_OUT_IPA=$EXPORT_DIR/General_${IPA_APP_NAME}App_Chn_AdHoc_IS_V${IPA_BUNDLE_VERSION}.R.${BUILD_DATE}.ipa

 resignFile "$REGISN_PROFILE_FILE" "$REGISN_KEYCHAI_ID" "${TEMP}/Payload/${APP_NAME}"

 #生成ipa文件
 echo "zip file generate new ipa file"
 cd $TEMP

 echo `zip -qr resign.ipa Payload`
 mv resign.ipa ${REGISN_OUT_IPA}

 echo "[******************** End Resign ipa for ad-hoc... ********************] \r\n \r\n"




#=============压缩=================
echo "[******************** Step5. Zip dSYM... ********************]"

cd $EXPORT_DIR
mkdir -p Release-iphoneos
cp -rf $ARCHIVE_PATH/dSYMs/${SCHEME_NAME}.app.dSYM $EXPORT_DIR/Release-iphoneos/${SCHEME_NAME}.app.dSYM
tar czf ${PROJECT_NAME}.tar.gz Release-iphoneos *.ipa
mv ${PROJECT_NAME}.tar.gz ${BUILD_SCRIPT_PATH}/${PROJECT_NAME}.tar.gz

echo "[******************** Zip Successful... ********************] \r\n \r\n"



echo "[******************** Over. Clean resources... ********************]"

cd ${PROJECT_DIR}
echoCommand "rm -rf ${EXPORT_DIR}"
echoCommand "rm -rf ${TEMP}"
echoCommand "rm -rf ${ENTITLEMENTS_PLIST}"
echoCommand "rm -rf ${APP_STORE_PLIST_FILE}"
echoCommand "rm -rf entitlements.plist"
echoCommand "rm -rf applestore_temp.plist"
#echoCommand "rm -rf ${ARCHIVE_PATH}"

cd ${BUILD_SCRIPT_PATH}
echo "[******************** Clean Successful... ********************]"


BUILD_ENDTIME=`date "+%Y-%m-%d %H:%M:%S"`
echo "*******************编译结束: ${BUILD_ENDTIME}"
BUILD_ENDTIME=`date +%s`
BUILD_SPENDTIME=`expr ${BUILD_ENDTIME} - ${BUILD_STARTTIME}`
BUILD_SPENDTIME_MIN=`expr ${BUILD_SPENDTIME} / 60`
BUILD_SPENDTIME_SEC=`expr ${BUILD_SPENDTIME} % 60`
echo "*******************耗时: ${BUILD_SPENDTIME_MIN}分${BUILD_SPENDTIME_SEC}秒"
